import time

from django.core.files.uploadedfile import SimpleUploadedFile
from django.core.urlresolvers import reverse
from django.utils import unittest
from django.template import Context, Template
from django.test.client import Client

from .models import Example


# simple 1-page pdf saying 'Hello, world!'
TEST_DOC_DATA = (  # Multiline string, not tuple.
    '%PDF-1.7\n\n1 0 obj  % entry point\n<<\n  /Type /Catalog\n  /Pages 2 0 '
    'R\n>>\nendobj\n\n2 0 obj\n<<\n  /Type /Pages\n  /MediaBox [ 0 0 200 200 '
    ']\n  /Count 1\n  /Kids [ 3 0 R ]\n>>\nendobj\n\n3 0 obj\n<<\n  /Type '
    '/Page\n  /Parent 2 0 R\n  /Resources <<\n    /Font <<\n      /F1 4 0 R \n'
    '    >>\n  >>\n  /Contents 5 0 R\n>>\nendobj\n\n4 0 obj\n<<\n  /Type '
    '/Font\n  /Subtype /Type1\n  /BaseFont /Times-Roman\n>>\nendobj\n\n5 0 obj'
    '  % page content\n<<\n  /Length 44\n>>\nstream\nBT\n70 50 TD\n/F1 12 '
    'Tf\n(Hello, world!) Tj\nET\nendstream\nendobj\n\nxref\n0 6\n0000000000 '
    '65535 f \n0000000010 00000 n \n0000000079 00000 n \n0000000173 00000 n '
    '\n0000000301 00000 n \n0000000380 00000 n \ntrailer\n<<\n  /Size 6\n  '
    '/Root 1 0 R\n>>\nstartxref\n492\n%%EOF\n'
)
TEST_DOC_NAME = 'test_doc_file.pdf'


client = Client()


def initial_setup():
    """ Inits all here as we do not want doing it in *every* test """
    # Create sample data
    example = Example.objects.create(name='Test item',
        document=SimpleUploadedFile(TEST_DOC_NAME, TEST_DOC_DATA))

    # Get data out of the model
    instance = Example.objects.get(id=example.id)
    return instance


class CrocoTestCase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.instance = initial_setup()

    def setUp(self):
        # there is a race conditions somewhere so sleep between each test
        time.sleep(1)

    def render(self, tmpl):
        tmplout = "{% load croco_tags %}{% autoescape off %}"
        tmplout += tmpl
        tmplout += "{% endautoescape %}"
        return Template(tmplout).render(Context({'obj': self.instance}))

    def assertContains(self, test_value, expected_set):
        # That assert method does not exist in Py2.6
        msg = "%s does not contain %s" % (test_value, expected_set)
        self.assert_(test_value not in expected_set, msg)

    def test_document_name(self):
        # Ensure document has correct name
        self.assertEqual(self.instance.document.name, TEST_DOC_NAME)

    def test_document_size(self):
        # Ensure correct size
        self.assertEqual(self.instance.document.size, 679)
        self.assertEqual(self.instance.document.size_human, '679 bytes')

    def test_document_type(self):
        # Ensure correct file type
        self.assertEqual(self.instance.document.type, 'pdf')

    def test_document_uuid(self):
        # Ensure correct length of UUID
        # UUID is 32 long chars, but there is 36 including dash chars
        uuid = self.instance.document.uuid
        self.assertEqual(len(uuid), 36)

    def test_document_thumbnail(self):
        # Ensure in-line image was returned
        self.assertContains(self.instance.document.thumbnail, 'data:image/png')

    def test_document_url(self):
        # Ensure correct URL was returned for `url`
        url = self.instance.document.url
        expected_url = reverse('croco_document_url',
            kwargs={'uuid': self.instance.document.uuid})
        self.assertEqual(url, expected_url)

        # Ensure correct redirect was made
        response = client.get(url)
        self.assertEqual(response.status_code, 302)
        self.assertContains(response._headers['location'][1], 'crocodoc.com')

    def test_document_url_with_annotations(self):
        tmpl = "{{ obj.document.url|annotated:'true' }}"
        response = client.get(self.render(tmpl))
        self.assertEqual(response.status_code, 302)
        self.assertContains(response._headers['location'][1], 'crocodoc.com')

    def test_document_url_with_editable_and_user(self):
        tmpl = "{{ obj.document.url|editable:'true'|user_id:'1'|user_name:'admin' }}"
        response = client.get(self.render(tmpl))
        self.assertEqual(response.status_code, 302)
        self.assertContains(response._headers['location'][1], 'crocodoc.com')

    def test_document_content_url(self):
        # Ensure correct URL for `content_url`
        content_url = self.instance.document.content_url
        expected_url = reverse('croco_document_content_url',
            kwargs={'uuid': self.instance.document.uuid})
        self.assertEqual(content_url, expected_url)

        # Ensure correct response
        response = client.get(content_url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response.content, 'crocodoc.com')

    def test_document_content_url_with_user_filter(self):
        tmpl = "{{ obj.document.content_url|user_filter:'1,2' }}"
        response = client.get(self.render(tmpl))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response.content, 'crocodoc.com')

    def test_document_download(self):
        # Ensure correct URL for `download_document`
        document_url = self.instance.document.download_document
        expected_url = reverse('croco_document_download',
            kwargs={'uuid': self.instance.document.uuid})
        self.assertEqual(document_url, expected_url)

        # Ensure correct response
        response = client.get(document_url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.content), 679)
        self.assertEqual(response._headers['content-type'][1],
            'application/pdf')

    def test_document_download_with_annotations(self):
        tmpl = "{{ obj.document.download_document|annotated:'true' }}"
        response = client.get(self.render(tmpl))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.content), 1049)
        self.assertEqual(response._headers['content-type'][1],
            'application/pdf')

    def test_thumbnail_download(self):
        # Ensure correct URL for `download_thumbnail`
        thumbnail_url = self.instance.document.download_thumbnail
        expected_url = reverse('croco_thumbnail_download',
            kwargs={'uuid': self.instance.document.uuid})
        self.assertEqual(thumbnail_url, expected_url)

        # Ensure correct response
        response = client.get(thumbnail_url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response._headers['content-type'][1], 'image/png')

    def test_thumbnail_download_with_custom_size(self):
        tmpl = "{{ obj.document.download_thumbnail|size:'121x121' }}"
        response = client.get(self.render(tmpl))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response._headers['content-type'][1], 'image/png')

    def test_text_download(self):
        # Ensure correct URL for `download_text`
        text_url = self.instance.document.download_text
        expected_url = reverse('croco_text_download',
            kwargs={'uuid': self.instance.document.uuid})
        self.assertEqual(text_url, expected_url)

        # Ensure text is not returned for test account (an account without
        # extracting text feature enabled)
        response = client.get(text_url)
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, '{"error": "text not available"}')



import urlparse
import urllib

from django import template

register = template.Library()


def add_query_params(url, params=None):
    """
    Adds additional query params to the given url, preserving original ones.

    Usage:
    >>> add_query_params('http://foo.com', {'a': 'b'})
    'http://foo.com?a=b'
    >>> add_query_params('http://foo.com/?a=b', {'b': 'c', 'd': 'q'})
    'http://foo.com/?a=b&b=c&d=q'
    """
    if not params:
        return url
    encoded = urllib.urlencode(params)
    url = urlparse.urlparse(url)
    return urlparse.urlunparse((url.scheme, url.netloc, url.path, url.params,
        (encoded if not url.query else url.query + '&' + encoded),
        url.fragment))


# TODO: dynamic creation of filters using python's closures?
@register.filter
def editable(url, editable):
    return add_query_params(url, {'editable': editable})


@register.filter
def user_id(url, user_id):
    return add_query_params(url, {'user_id': user_id})


@register.filter
def user_name(url, user_name):
    return add_query_params(url, {'user_name': user_name})


@register.filter
def user_filter(url, user_filter):
    return add_query_params(url, {'filter': user_filter})


@register.filter
def admin(url, admin):
    return add_query_params(url, {'admin': admin})


@register.filter
def downloadable(url, downloadable):
    return add_query_params(url, {'downloadable': downloadable})


@register.filter
def copyprotected(url, copyprotected):
    return add_query_params(url, {'copyprotected': copyprotected})


@register.filter
def demo(url, demo):
    return add_query_params(url, {'demo': demo})


@register.filter
def sidebar(url, sidebar):
    return add_query_params(url, {'sidebar': sidebar})


@register.filter
def pdf(url, pdf):
    return add_query_params(url, {'pdf': pdf})


@register.filter
def filename(url, filename):
    return add_query_params(url, {'filename': filename})


@register.filter
def annotated(url, annotated):
    return add_query_params(url, {'annotated': annotated})


@register.filter
def size(url, size):
    return add_query_params(url, {'size': size})



import crocodoc

from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.views.generic import View


class CrocoDocumentView(View):
    redirect = None

    def get(self, request, *args, **kwargs):
        uuid = kwargs.pop('uuid', None)
        if uuid is None:
            raise Http404

        try:
            params = {}
            qs_params = request.GET

            bool_params = ("editable", "admin", "downloadable", "copyprotected", "demo")
            for p in bool_params:
                if p in qs_params:
                    params[p] = True

            if 'user_id' in qs_params and 'user_name' in qs_params:
                params['user'] = {
                    'id': qs_params['user_id'],
                    'name': qs_params['user_name'],
                }

            if 'filter' in qs_params:
                params['filter'] = qs_params['filter']

            if 'sidebar' in qs_params:
                params['sidebar'] = qs_params['sidebar']

            session = crocodoc.session.create(uuid, **params)
        except crocodoc.CrocodocError as e:
            return HttpResponse(content=e.response_content,
                status=e.status_code)

        url = 'https://crocodoc.com/view/{0}'.format(session)

        if self.redirect:
            return HttpResponseRedirect(url)
        return HttpResponse(content=url)


class CrocoDocumentDownload(View):
    """
    Downloads document from Crocodoc in PDF format.
    TODO: allow to download original version
    """
    def get(self, request, *args, **kwargs):
        uuid = kwargs.pop('uuid', None)
        if uuid is None:
            raise Http404

        try:
            qs_params = request.GET
            pdf = True
            annotated = filter_by = None
            if 'annotated' in qs_params and qs_params['annotated'].lower() == 'true':
                annotated = True
            if 'filter' in qs_params:
                filter_by = True
            file = crocodoc.download.document(uuid, pdf=pdf, annotated=annotated,
                user_filter=filter_by)
        except crocodoc.CrocodocError as e:
            return HttpResponse(content=e.response_content,
                status=e.status_code)

        response = HttpResponse(mimetype='application/pdf')
        response['Content-Disposition'] = 'attachment; filename=%s.pdf' % uuid
        response.write(file)
        return response


class CrocoThumbnailDownload(View):
    def get(self, request, *args, **kwargs):
        uuid = kwargs.pop('uuid', None)
        if uuid is None:
            raise Http404

        try:
            width = height = 100
            if 'size' in request.GET:
                width, height = request.GET['size'].split('x')
            image = crocodoc.download.thumbnail(uuid, width=int(width),
                height=int(height))
        except crocodoc.CrocodocError as e:
            return HttpResponse(content=e.response_content,
                status=e.status_code)

        response = HttpResponse(mimetype='image/png')
        response['Content-Disposition'] = 'attachment; filename=%s.png' % uuid
        response.write(image)
        return response


class CrocoTextDownload(View):
    def get(self, request, *args, **kwargs):
        uuid = kwargs.pop('uuid', None)
        if uuid is None:
            raise Http404

        try:
            text = crocodoc.download.text(uuid)
        except crocodoc.CrocodocError as e:
            return HttpResponse(content=e.response_content,
                status=e.status_code)

        return HttpResponse(content=text)



import base64
import json
import os

from django import forms, get_version
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.files.storage import Storage
from django.core.urlresolvers import reverse
from django.db import models
from django.template.defaultfilters import filesizeformat
from django.utils.translation import ugettext_lazy as _

if get_version()[:3] == '1.3':
    string_types = basestring
else:
    from django.utils.six import string_types

import crocodoc
from crocodoc import CrocodocError

_token = 'CROCO_API_TOKEN'
CROCO_API_TOKEN = getattr(settings, _token, os.environ.get(_token))
if CROCO_API_TOKEN is None:
    raise ImproperlyConfigured("CROCO_API_TOKEN settings is required."
        " Define it in your project's settings.py or env.sh file.")

crocodoc.api_token = CROCO_API_TOKEN


class CrocoStorage(Storage):
    def __init__(self):
        self._croco_uuid = None

    def get_valid_name(self, name):
        from django.utils.text import get_valid_filename
        return get_valid_filename(name)

    def _save(self, file):
        try:
            uuid = crocodoc.document.upload(file=file)
            setattr(self, '_croco_uuid', uuid)
        except CrocodocError as croco_error:
            raise croco_error

        return uuid


class CrocoFieldObject(object):
    def __init__(self, instance, attrs):
        self.instance = instance
        self.attrs = attrs

    def __getattr__(self, name):
        if name in self.attrs:
            return self.attrs[name]
        return name

    @property
    def size_human(self):
        return filesizeformat(self.attrs['size'])

    @property
    def thumbnail(self):
        return self.instance._get_thumbnail(self.attrs['uuid'])

    @property
    def url(self):
        return self._url_for('croco_document_url')

    @property
    def content_url(self):
        return self._url_for('croco_document_content_url')

    @property
    def download_document(self):
        return self._url_for('croco_document_download')

    @property
    def download_thumbnail(self):
        return self._url_for('croco_thumbnail_download')

    @property
    def download_text(self):
        return self._url_for('croco_text_download')

    def _url_for(self, url):
        return reverse(url, kwargs={'uuid': self.attrs['uuid']})

    def __unicode__(self):
        return "%s" % self.attrs['name']

    def __str__(self):
        return "%s" % self.attrs['name']


class CrocoField(models.Field):
    __metaclass__ = models.SubfieldBase
    description = _("CrocoField")

    def __init__(self, verbose_name=None, name=None, *args, **kwargs):
        self.storage = CrocoStorage()
        self.thumbnail_size = kwargs.pop('thumbnail_size', (100, 100))
        super(CrocoField, self).__init__(*args, **kwargs)

    def get_internal_type(self):
        return "TextField"

    def to_python(self, value):
        if value == "":
            return None

        try:
            if isinstance(value, string_types):
                return CrocoFieldObject(self, json.loads(value))
        except ValueError:
            raise
        return value

    def pre_save(self, model_instance, add):
        value = super(CrocoField, self).pre_save(model_instance, add)
        if not isinstance(value, CrocoFieldObject):
            croco_uuid = self.storage._save(value)
            file_attrs = {
                'name': value.name,
                'size': value.size,
                'uuid': croco_uuid,
                'type': self._file_ext(value.name),
            }
            value = CrocoFieldObject(self, file_attrs)
        return self.get_prep_value(value)

    def get_prep_value(self, value):
        if isinstance(value, CrocoFieldObject):
            return json.dumps(value.attrs)
        return value

    def clean(self, instance, filename):
        if not self._is_document(instance.name):
            raise forms.ValidationError("Unsupported file type.")
        return super(CrocoField, self).clean(instance, filename)

    def formfield(self, **kwargs):
        defaults = {'form_class': forms.FileField}
        defaults.update(kwargs)
        return super(CrocoField, self).formfield(**defaults)

    def _get_thumbnail(self, uuid):
        """
        Returns in-line image using URI scheme.
        TODO: add support for custom image field (so thumbnails are cached)
        """
        try:
            status = crocodoc.document.status(uuid)
            if status.get('error') is None:
                try:
                    attrs = {
                        'width': self.thumbnail_size[0],
                        'height': self.thumbnail_size[1],
                    }
                    thumb = crocodoc.download.thumbnail(uuid, **attrs)
                    return "data:image/png;base64," + base64.b64encode(thumb)
                except CrocodocError as e:
                    return e.error_message
            else:
                return status.get('error')
        except CrocodocError as e:
            return e.error_message

    def _file_ext(self, filename):
        """ Return an extension of the file """
        return os.path.splitext(filename)[1][1:]

    def _is_document(self, filename):
        allowed_exts = ('pdf', 'ppt', 'pptx', 'doc', 'docx', 'xls', 'xlsx',
            'csv')
        _root, ext = os.path.splitext(filename.lower())
        if ext[1:] in allowed_exts:
            return True
        return False

try:
    from south.modelsinspector import add_introspection_rules
except ImportError:
    pass
else:
    add_introspection_rules([], ["^djcroco\.fields\.CrocoField"])



from django import get_version

if get_version()[:3] == '1.3':
    from django.conf.urls.defaults import patterns, include, url
else:
    from django.conf.urls import patterns, include, url


import views

urlpatterns = patterns('',
    url(r'^croco_document/(?P<uuid>[-\w]+)$',
        views.CrocoDocumentView.as_view(redirect=True),
        name='croco_document_url'),
    url(r'^croco_document_content/(?P<uuid>[-\w]+)$',
        views.CrocoDocumentView.as_view(),
        name='croco_document_content_url'),
    url(r'^croco_document_download/(?P<uuid>[-\w]+)$',
        views.CrocoDocumentDownload.as_view(),
        name='croco_document_download'),
    url(r'^croco_thumbnail_download/(?P<uuid>[-\w]+)$',
        views.CrocoThumbnailDownload.as_view(),
        name='croco_thumbnail_download'),
    url(r'^croco_text_download/(?P<uuid>[-\w]+)$',
        views.CrocoTextDownload.as_view(),
        name='croco_text_download'),
)



from django.conf.global_settings import TEMPLATE_CONTEXT_PROCESSORS

DEBUG = True

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

TEMPLATE_DEBUG = DEBUG
TEMPLATE_CONTEXT_PROCESSORS += (
    'django.core.context_processors.request',
)

SECRET_KEY = 'lolz'

INSTALLED_APPS = (
    'djcroco.tests',
    'djcroco',
)

ROOT_URLCONF = 'djcroco.tests.urls'



from setuptools import setup, find_packages

import djcroco


setup(
    name='djcroco',
    packages=find_packages(),
    include_package_data=True,
    version=djcroco.__version__,
    description='',
    long_description=open('README.rst').read(),
    author=djcroco.__author__,
    author_email='matt.lenc@gmail.com',
    url='https://github.com/mattack108/djcroco/',
    install_requires=[
        'crocodoc',
    ],
    zip_safe=False,
)



from django import get_version

if get_version()[:3] == '1.3':
    from django.conf.urls.defaults import patterns, include, url
else:
    from django.conf.urls import patterns, include, url


urlpatterns = patterns('',
    url(r'', include('djcroco.urls')),
)



from django.db import models

from djcroco.fields import CrocoField


class Example(models.Model):
    name = models.CharField(max_length=255)
    document = CrocoField()

    def __unicode__(self):
        return self.name
